#include "clang/StaticAnalyzer/Core/BugReporter/BugType.h"
#include "clang/StaticAnalyzer/Core/Checker.h"
#include "clang/StaticAnalyzer/Core/PathSensitive/CallEvent.h"
#include "clang/StaticAnalyzer/Core/PathSensitive/CheckerContext.h"
#include "clang/StaticAnalyzer/Core/CheckerRegistry.h"


//To check whether the checker registered: clang -cc1 -load ./libMSC.so -analyzer-checker-help
//To run the checker: clang -Xclang -load -Xclang ./libMSC.so --analyze -Xclang -analyzer-checker=example.mycheck test.c

using namespace clang;
using namespace ento;

namespace {
  typedef SmallVector<SymbolRef, 2> SymbolVector;
  
  struct StreamState {
  private:
    enum Kind { Opened, Closed } K;
    
    StreamState(Kind InK) : K(InK) { }
    
  public:
    bool operator==(const StreamState &X) const {
      return K == X.K;
    }
    void Profile(llvm::FoldingSetNodeID &ID) const {
      ID.AddInteger(K);
    }
    
    bool isOpened() const { return K == Opened; }
    bool isClosed() const { return K == Closed; }
    
    static StreamState getOpened() { return StreamState(Opened); }
    static StreamState getClosed() { return StreamState(Closed); }    
  };
  
  class MyStreamChecker : public Checker<check::PostCall> {
    
    std::unique_ptr<BugType> DoubleCloseBugType;
    std::unique_ptr<BugType> LeakBugType;
    
  public:
    void checkPostCall(const CallEvent &Call, CheckerContext &C) const;
    MyStreamChecker();
  };
  
} // end anonymous namespace

MyStreamChecker::MyStreamChecker() {
  // Initialize the bug types.
  DoubleCloseBugType.reset(
			   new BugType(this, "Double fclose", "Unix Stream API Error"));
  
  LeakBugType.reset(
		    new BugType(this, "Resource Leak", "Unix Stream API Error"));
  // Sinks are higher importance bugs as well as calls to assert() or exit(0).
  LeakBugType->setSuppressOnSink(true);
}

REGISTER_MAP_WITH_PROGRAMSTATE(StreamMap, SymbolRef, StreamState)

void MyStreamChecker::checkPostCall(const CallEvent &Call,
				    CheckerContext &C) const {

  
}



extern "C"
void clang_registerCheckers (CheckerRegistry &registry) {
  registry.addChecker<MyStreamChecker>("example.mycheck", "Stream API checker");
}

extern "C"
const char clang_analyzerAPIVersionString[] = CLANG_ANALYZER_API_VERSION_STRING;
